/*
!===============================================================================
!
! machine_info.c
!
!-------------------------------------------------------------------------------
!
! Copyright (C) 2014 Hinnerk Stueben
!
! This file is part of BQCD -- Berlin Quantum ChromoDynamics program
!
! BQCD is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! BQCD is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with BQCD.  If not, see <http://www.gnu.org/licenses/>.
!
!-------------------------------------------------------------------------------
*/
# define _GNU_SOURCE
# include <unistd.h>
# include <sys/syscall.h>
# include <stdio.h>
# include <string.h>
#ifdef QUDA_SOLVER
# include <cuda_runtime.h>
#endif

int num_processors(void);
int my_processor(void);
int num_physical_ids(void);
int my_physical_id(void);
int num_cuda_devices(void);

int num_processors(void) // number of processors from /proc/cpuinfo
{
    char word[1024];
    FILE *fp;
    int count = 0;

    fp = fopen("/proc/cpuinfo", "r");

    while (fscanf(fp, "%s", word) != EOF) {
        if ( ! strcmp(word, "processor")) count++;
    }
    return count;
}

int my_processor(void) // ID of the processor currently executing this thread
{
    char procstat[128];
    FILE *fp;
    int tid, processor;

    tid = syscall(SYS_gettid);
    sprintf(procstat, "/proc/%d/stat", tid);
    fp = fopen(procstat, "r");
    fscanf(fp, 
	   "%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s "
	   "%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s "
	   "%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s "
	   "%*s %*s %*s %*s %*s %*s %*s %*s %d", &processor);
    fclose(fp);
    return processor;
}

int num_physical_ids(void) // number of physical IDs (CPUs) from /proc/cpuinfo
{
    char word[1024];
    FILE *fp;
    int found_physical = 0;
    int num_ids = 0;
    int id;

    fp = fopen("/proc/cpuinfo", "r");

    while (fscanf(fp, "%s", word) != EOF) {
        if (found_physical) {
	    if ( ! strcmp(word, "id")) {
                fscanf(fp, "%*s %d", &id);
                if (id > num_ids) num_ids = id;
	    }            
	    found_physical = 0;
        } else { 
            if ( ! strcmp(word, "physical")) found_physical = 1;
        }
    }
    return num_ids + 1;
}

int my_physical_id(void) // physical ID (CPU) of this thread
{
    char word[1024];
    FILE *fp;
    int processor;
    int id;
    int count = 0;
    int found_processor = 0;
    int found_physical = 0;
    int me = my_processor();

    fp = fopen("/proc/cpuinfo", "r");

    while (fscanf(fp, "%s", word) != EOF) {
        if (found_processor) {
            if (found_physical) {
	        if ( ! strcmp(word, "id")) {
                    fscanf(fp, "%*s %d", &id);
                    return id;
	        }            
	        found_physical = 0;
            } else {
                if ( ! strcmp(word, "physical")) found_physical = 1;
            }
        } else {
            if ( ! strcmp(word, "processor")) { 
                fscanf(fp, "%*s %d", &processor);
                if (processor == me) found_processor = 1;
            }
        }
    }
    return -1;
}

int num_cuda_devices(void) // number of CUDA devices
{
#ifdef QUDA_SOLVER
    int deviceCount;
    cudaGetDeviceCount(&deviceCount);
    return deviceCount;
#else
    return 0;
#endif
}
